@@ -63,6 +63,7 @@ struct _CdDevicePrivate
 	gchar			*serial;
 	gchar			*format;
 	gchar			*vendor;
+	gchar			**profiling_inhibitors;
 	guint64			 created;
 	guint64			 modified;
 	GPtrArray		*profiles;
@@ -90,6 +91,7 @@ enum {
 	PROP_MODE,
 	PROP_SCOPE,
 	PROP_OWNER,
+	PROP_PROFILING_INHIBITORS,
 	PROP_LAST
 };
 
@@ -225,6 +227,24 @@ cd_device_get_format (CdDevice *device)
 	return device->priv->format;
 }
 
+/**
+ * cd_device_get_profiling_inhibitors:
+ * @device: a #CdDevice instance.
+ *
+ * Gets any profiling inhibitors for the device.
+ *
+ * Return value: (transfer none): A strv, or %NULL for invalid
+ *
+ * Since: 0.1.17
+ **/
+const gchar **
+cd_device_get_profiling_inhibitors (CdDevice *device)
+{
+	g_return_val_if_fail (CD_IS_DEVICE (device), NULL);
+	g_return_val_if_fail (device->priv->proxy != NULL, NULL);
+	return (const gchar **) device->priv->profiling_inhibitors;
+}
+
 /**
  * cd_device_get_created:
  * @device: a #CdDevice instance.
@@ -404,12 +424,11 @@ cd_device_set_profiles_array_from_variant (CdDevice *device,
 	gchar *object_path_tmp;
 	gsize len;
 	guint i;
-	GVariantIter iter;
 
 	g_ptr_array_set_size (device->priv->profiles, 0);
 	if (profiles == NULL)
 		goto out;
-	len = g_variant_iter_init (&iter, profiles);
+	len = g_variant_n_children (profiles);
 	for (i=0; i<len; i++) {
 		g_variant_get_child (profiles, i,
 				     "o", &object_path_tmp);
@@ -464,7 +483,7 @@ cd_device_get_metadata_item (CdDevice *device, const gchar *key)
 static void
 cd_device_set_metadata_from_variant (CdDevice *device, GVariant *variant)
 {
-	GVariantIter *iter = NULL;
+	GVariantIter iter;
 	const gchar *prop_key;
 	const gchar *prop_value;
 
@@ -472,9 +491,8 @@ cd_device_set_metadata_from_variant (CdDevice *device, GVariant *variant)
 	g_hash_table_remove_all (device->priv->metadata);
 
 	/* insert the new metadata */
-	g_variant_get (variant, "a{ss}",
-		       &iter);
-	while (g_variant_iter_loop (iter, "{ss}",
+	g_variant_iter_init (&iter, variant);
+	while (g_variant_iter_loop (&iter, "{ss}",
 				    &prop_key, &prop_value)) {
 		g_hash_table_insert (device->priv->metadata,
 				     g_strdup (prop_key),
@@ -483,6 +501,23 @@ cd_device_set_metadata_from_variant (CdDevice *device, GVariant *variant)
 	}
 }
 
+/**
+ * cd_device_get_nullable_str:
+ *
+ * We can't get nullable types from a GVariant yet. Work around...
+ **/
+static gchar *
+cd_device_get_nullable_str (GVariant *value)
+{
+	const gchar *tmp;
+	tmp = g_variant_get_string (value, NULL);
+	if (tmp == NULL)
+		return NULL;
+	if (tmp[0] == '\0')
+		return NULL;
+	return g_strdup (tmp);
+}
+
 /**
  * cd_device_dbus_properties_changed_cb:
  **/
@@ -508,16 +543,19 @@ cd_device_dbus_properties_changed_cb (GDBusProxy  *proxy,
 				     &property_value);
 		if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_MODEL) == 0) {
 			g_free (device->priv->model);
-			device->priv->model = g_variant_dup_string (property_value, NULL);
+			device->priv->model = cd_device_get_nullable_str (property_value);
 		} else if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_SERIAL) == 0) {
 			g_free (device->priv->serial);
-			device->priv->serial = g_variant_dup_string (property_value, NULL);
+			device->priv->serial = cd_device_get_nullable_str (property_value);
 		} else if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_FORMAT) == 0) {
 			g_free (device->priv->format);
-			device->priv->format = g_variant_dup_string (property_value, NULL);
+			device->priv->format = cd_device_get_nullable_str (property_value);
 		} else if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_VENDOR) == 0) {
 			g_free (device->priv->vendor);
-			device->priv->vendor = g_variant_dup_string (property_value, NULL);
+			device->priv->vendor = cd_device_get_nullable_str (property_value);
+		} else if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_PROFILING_INHIBITORS) == 0) {
+			g_free (device->priv->profiling_inhibitors);
+			device->priv->profiling_inhibitors = g_variant_dup_strv (property_value, NULL);
 		} else if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_KIND) == 0) {
 			device->priv->kind =
 				cd_device_kind_from_string (g_variant_get_string (property_value, NULL));
@@ -536,6 +574,8 @@ cd_device_dbus_properties_changed_cb (GDBusProxy  *proxy,
 			device->priv->modified = g_variant_get_uint64 (property_value);
 		} else if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_METADATA) == 0) {
 			cd_device_set_metadata_from_variant (device, property_value);
+		} else if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_OWNER) == 0) {
+			device->priv->owner = g_variant_get_uint32 (property_value);
 		} else if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_ID) == 0) {
 			/* ignore this, we don't support it changing */;
 		} else {
@@ -614,6 +654,7 @@ cd_device_connect_cb (GObject *source_object,
 	GVariant *serial = NULL;
 	GVariant *format = NULL;
 	GVariant *vendor = NULL;
+	GVariant *profiling_inhibitors = NULL;
 	GVariant *colorspace = NULL;
 	GVariant *scope = NULL;
 	GVariant *owner = NULL;
@@ -641,7 +682,7 @@ cd_device_connect_cb (GObject *source_object,
 	id = g_dbus_proxy_get_cached_property (device->priv->proxy,
 					       CD_DEVICE_PROPERTY_ID);
 	if (id != NULL)
-		device->priv->id = g_variant_dup_string (id, NULL);
+		device->priv->id = cd_device_get_nullable_str (id);
 
 	/* if the device is missing, then fail */
 	if (id == NULL) {
@@ -691,25 +732,31 @@ cd_device_connect_cb (GObject *source_object,
 	model = g_dbus_proxy_get_cached_property (device->priv->proxy,
 						  CD_DEVICE_PROPERTY_MODEL);
 	if (model != NULL)
-		device->priv->model = g_variant_dup_string (model, NULL);
+		device->priv->model = cd_device_get_nullable_str (model);
 
 	/* get serial */
 	serial = g_dbus_proxy_get_cached_property (device->priv->proxy,
 						   CD_DEVICE_PROPERTY_SERIAL);
 	if (serial != NULL)
-		device->priv->serial = g_variant_dup_string (serial, NULL);
+		device->priv->serial = cd_device_get_nullable_str (serial);
 
 	/* get format */
 	format = g_dbus_proxy_get_cached_property (device->priv->proxy,
 						   CD_DEVICE_PROPERTY_FORMAT);
 	if (format != NULL)
-		device->priv->format = g_variant_dup_string (format, NULL);
+		device->priv->format = cd_device_get_nullable_str (format);
 
 	/* get vendor */
 	vendor = g_dbus_proxy_get_cached_property (device->priv->proxy,
 						   CD_DEVICE_PROPERTY_VENDOR);
 	if (vendor != NULL)
-		device->priv->vendor = g_variant_dup_string (vendor, NULL);
+		device->priv->vendor = cd_device_get_nullable_str (vendor);
+
+	/* get profiling inhibitors */
+	profiling_inhibitors = g_dbus_proxy_get_cached_property (device->priv->proxy,
+								 CD_DEVICE_PROPERTY_PROFILING_INHIBITORS);
+	if (profiling_inhibitors != NULL)
+		device->priv->profiling_inhibitors = g_variant_dup_strv (profiling_inhibitors, NULL);
 
 	/* get created */
 	created = g_dbus_proxy_get_cached_property (device->priv->proxy,
@@ -755,6 +802,8 @@ out:
 		g_variant_unref (model);
 	if (vendor != NULL)
 		g_variant_unref (vendor);
+	if (profiling_inhibitors != NULL)
+		g_variant_unref (profiling_inhibitors);
 	if (serial != NULL)
 		g_variant_unref (serial);
 	if (format != NULL)
@@ -1819,6 +1868,9 @@ cd_device_get_property (GObject *object, guint prop_id, GValue *value, GParamSpe
 	case PROP_VENDOR:
 		g_value_set_string (value, device->priv->vendor);
 		break;
+	case PROP_PROFILING_INHIBITORS:
+		g_value_set_boxed (value, device->priv->profiling_inhibitors);
+		break;
 	case PROP_KIND:
 		g_value_set_uint (value, device->priv->kind);
 		break;
@@ -1983,6 +2035,19 @@ cd_device_class_init (CdDeviceClass *klass)
 							      NULL, NULL,
 							      NULL,
 							      G_PARAM_READABLE));
+	/**
+	 * CdDevice:profiling_inhibitors:
+	 *
+	 * Any profiling inhibitors.
+	 *
+	 * Since: 0.1.17
+	 **/
+	g_object_class_install_property (object_class,
+					 PROP_PROFILING_INHIBITORS,
+					 g_param_spec_boxed ("profiling-inhibitors",
+							      NULL, NULL,
+							      G_TYPE_STRV,
+							      G_PARAM_READABLE));
 	/**
 	 * CdDevice:kind:
 	 *
@@ -2099,6 +2164,7 @@ cd_device_finalize (GObject *object)
 	g_free (device->priv->serial);
 	g_free (device->priv->format);
 	g_free (device->priv->vendor);
+	g_strfreev (device->priv->profiling_inhibitors);
 	g_ptr_array_unref (device->priv->profiles);
 	if (device->priv->proxy != NULL) {
 		ret = g_signal_handlers_disconnect_by_func (device->priv->proxy,
