@@ -33,6 +33,7 @@
 #include "cd-color.h"
 #include "cd-device.h"
 #include "cd-device-sync.h"
+#include "cd-it8.h"
 #include "cd-math.h"
 #include "cd-profile.h"
 #include "cd-profile-sync.h"
@@ -99,6 +100,181 @@ _g_test_realpath (const gchar *relpath)
 
 /**********************************************************************/
 
+static void
+colord_it8_raw_func (void)
+{
+	CdColorRGB rgb;
+	CdColorXYZ xyz;
+	CdIt8 *it8;
+	gboolean ret;
+	gchar *filename;
+	GError *error = NULL;
+	GFile *file;
+	GFile *file_new;
+
+	it8 = cd_it8_new ();
+	g_assert (it8 != NULL);
+
+	/* load in file */
+	filename = _g_test_realpath (TESTDATADIR "/raw.ti3");
+	file = g_file_new_for_path (filename);
+	ret = cd_it8_load_from_file (it8, file, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
+	/* write this to a new file */
+	file_new = g_file_new_for_path ("/tmp/test.ti3");
+	ret = cd_it8_save_to_file (it8, file_new, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
+	/* load in file again to ensure we save all the required data */
+	ret = cd_it8_load_from_file (it8, file_new, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
+	/* test values */
+	g_assert_cmpint (cd_it8_get_kind (it8), ==, CD_IT8_KIND_TI3);
+	g_assert_cmpint (cd_it8_get_data_size (it8), ==, 5);
+	g_assert (!cd_it8_get_normalized (it8));
+	g_assert_cmpstr (cd_it8_get_originator (it8), ==, "cd-self-test");
+	g_assert (!cd_it8_get_spectral (it8));
+	g_assert_cmpstr (cd_it8_get_instrument (it8), ==, "huey");
+	ret = cd_it8_get_data_item (it8, 1, &rgb, &xyz);
+	g_assert (ret);
+	g_assert_cmpfloat (ABS (rgb.R - 1.0f), <, 0.01f);
+	g_assert_cmpfloat (ABS (rgb.G - 1.0f), <, 0.01f);
+	g_assert_cmpfloat (ABS (rgb.B - 1.0f), <, 0.01f);
+	g_assert_cmpfloat (ABS (xyz.X - 145.46f), <, 0.01f);
+	g_assert_cmpfloat (ABS (xyz.Y - 99.88f), <, 0.01f);
+	g_assert_cmpfloat (ABS (xyz.Z - 116.59f), <, 0.01f);
+
+	/* remove temp file */
+	ret = g_file_delete (file_new, NULL, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
+	g_free (filename);
+	g_object_unref (it8);
+	g_object_unref (file);
+	g_object_unref (file_new);
+}
+
+static void
+colord_it8_normalized_func (void)
+{
+	CdColorRGB rgb;
+	CdColorXYZ xyz;
+	CdIt8 *it8;
+	gboolean ret;
+	gchar *filename;
+	GError *error = NULL;
+	GFile *file;
+	GFile *file_new;
+
+	it8 = cd_it8_new ();
+	g_assert (it8 != NULL);
+
+	/* load in file */
+	filename = _g_test_realpath (TESTDATADIR "/normalised.ti3");
+	file = g_file_new_for_path (filename);
+	ret = cd_it8_load_from_file (it8, file, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
+	/* write this to a new file */
+	file_new = g_file_new_for_path ("/tmp/test.ti3");
+	ret = cd_it8_save_to_file (it8, file_new, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
+	/* load in file again to ensure we save all the required data */
+	ret = cd_it8_load_from_file (it8, file_new, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
+	/* test values */
+	g_assert_cmpint (cd_it8_get_data_size (it8), ==, 2);
+	g_assert (!cd_it8_get_normalized (it8));
+	g_assert_cmpstr (cd_it8_get_originator (it8), ==, NULL);
+	g_assert (!cd_it8_get_spectral (it8));
+	g_assert_cmpstr (cd_it8_get_instrument (it8), ==, NULL);
+	ret = cd_it8_get_data_item (it8, 1, &rgb, &xyz);
+	g_assert (ret);
+	g_assert_cmpfloat (ABS (rgb.R - 1.0f), <, 0.01f);
+	g_assert_cmpfloat (ABS (rgb.G - 1.0f), <, 0.01f);
+	g_assert_cmpfloat (ABS (rgb.B - 1.0f), <, 0.01f);
+	g_assert_cmpfloat (ABS (xyz.X - 90.21f), <, 0.01f);
+	g_assert_cmpfloat (ABS (xyz.Y - 41.22f), <, 0.01f);
+	g_assert_cmpfloat (ABS (xyz.Z - 56.16f), <, 0.01f);
+
+	/* remove temp file */
+	ret = g_file_delete (file_new, NULL, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
+	g_free (filename);
+	g_object_unref (it8);
+	g_object_unref (file);
+	g_object_unref (file_new);
+}
+
+static void
+colord_it8_ccmx_func (void)
+{
+	CdIt8 *it8;
+	const CdMat3x3 *matrix;
+	gboolean ret;
+	gchar *filename;
+	GError *error = NULL;
+	GFile *file;
+	GFile *file_new;
+
+	it8 = cd_it8_new ();
+	g_assert (it8 != NULL);
+
+	/* load in file */
+	filename = _g_test_realpath (TESTDATADIR "/calibration.ccmx");
+	file = g_file_new_for_path (filename);
+	ret = cd_it8_load_from_file (it8, file, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
+	/* write this to a new file */
+	file_new = g_file_new_for_path ("/tmp/test.ccmx");
+	ret = cd_it8_save_to_file (it8, file_new, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
+	/* load in file again to ensure we save all the required data */
+	ret = cd_it8_load_from_file (it8, file_new, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
+	/* test values */
+	g_assert_cmpint (cd_it8_get_data_size (it8), ==, 0);
+	g_assert_cmpstr (cd_it8_get_originator (it8), ==, "cd-self-test");
+	g_assert_cmpstr (cd_it8_get_title (it8), ==, "Factory Calibration");
+	g_assert (!cd_it8_get_spectral (it8));
+	g_assert (cd_it8_has_option (it8, "TYPE_FACTORY"));
+	g_assert (!cd_it8_has_option (it8, "TYPE_XXXXXXX"));
+	g_assert_cmpstr (cd_it8_get_instrument (it8), ==, "Huey");
+	matrix = cd_it8_get_matrix (it8);
+	g_assert_cmpfloat (ABS (matrix->m00 - 1.3139f), <, 0.01f);
+	g_assert_cmpfloat (ABS (matrix->m01 - 0.21794f), <, 0.01f);
+	g_assert_cmpfloat (ABS (matrix->m02 - 0.89224f), <, 0.01f);
+
+	/* remove temp file */
+	ret = g_file_delete (file_new, NULL, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
+	g_free (filename);
+	g_object_unref (it8);
+	g_object_unref (file);
+	g_object_unref (file_new);
+}
+
 static void
 colord_client_get_devices_cb (GObject *object,
 			      GAsyncResult *res,
@@ -2021,6 +2197,9 @@ main (int argc, char **argv)
 	/* tests go here */
 	g_test_add_func ("/colord/color", colord_color_func);
 	g_test_add_func ("/colord/math", cd_test_math_func);
+	g_test_add_func ("/colord/it8-raw", colord_it8_raw_func);
+	g_test_add_func ("/colord/it8-normalized", colord_it8_normalized_func);
+	g_test_add_func ("/colord/it8-ccmx", colord_it8_ccmx_func);
 	g_test_add_func ("/colord/device", colord_device_func);
 	g_test_add_func ("/colord/client", colord_client_func);
 	g_test_add_func ("/colord/device-duplicate", colord_device_duplicate_func);
