@@ -25,6 +25,7 @@
 #include <gio/gio.h>
 #include <sys/time.h>
 #include <string.h>
+#include <pwd.h>
 
 #include "cd-common.h"
 #include "cd-device.h"
@@ -126,6 +127,16 @@ cd_device_set_scope (CdDevice *device, CdObjectScope object_scope)
 	device->priv->object_scope = object_scope;
 }
 
+/**
+ * cd_device_get_owner:
+ **/
+guint
+cd_device_get_owner (CdDevice *device)
+{
+	g_return_val_if_fail (CD_IS_DEVICE (device), G_MAXUINT);
+	return device->priv->owner;
+}
+
 /**
  * cd_device_set_owner:
  **/
@@ -223,25 +234,50 @@ cd_device_get_kind (CdDevice *device)
 	return device->priv->kind;
 }
 
+/**
+ * cd_device_set_object_path:
+ **/
+static void
+cd_device_set_object_path (CdDevice *device)
+{
+	gchar *path_tmp;
+	gchar *path_owner;
+	struct passwd *pw;
+
+	/* make sure object path is sane */
+	path_tmp = cd_main_ensure_dbus_path (device->priv->id);
+
+	/* append the uid to the object path */
+	pw = getpwuid (device->priv->owner);
+	if (device->priv->owner == 0 ||
+	    g_strcmp0 (pw->pw_name, DAEMON_USER) == 0) {
+		path_owner = g_strdup (path_tmp);
+	} else {
+		path_owner = g_strdup_printf ("%s_%s",
+					      path_tmp,
+					      pw->pw_name);
+	}
+	device->priv->object_path = g_build_filename (COLORD_DBUS_PATH,
+						      "devices",
+						      path_owner,
+						      NULL);
+	g_free (path_owner);
+	g_free (path_tmp);
+}
+
 /**
  * cd_device_set_id:
  **/
 void
 cd_device_set_id (CdDevice *device, const gchar *id)
 {
-	gchar *id_tmp;
-
 	g_return_if_fail (CD_IS_DEVICE (device));
-	g_free (device->priv->id);
 
-	/* make sure object path is sane */
-	id_tmp = cd_main_ensure_dbus_path (id);
-	device->priv->object_path = g_build_filename (COLORD_DBUS_PATH,
-						      "devices",
-						      id_tmp,
-						      NULL);
+	g_free (device->priv->id);
 	device->priv->id = g_strdup (id);
-	g_free (id_tmp);
+
+	/* now calculate this again */
+	cd_device_set_object_path (device);
 }
 
 /**
@@ -1326,6 +1362,7 @@ cd_device_dbus_get_property (GDBusConnection *connection_, const gchar *sender,
 	CdDevice *device = CD_DEVICE (user_data);
 	CdDevicePrivate *priv = device->priv;
 	gboolean ret;
+	gchar **bus_names = NULL;
 	GVariant *retval = NULL;
 
 	g_debug ("CdDevice %s:GetProperty '%s'",
@@ -1394,9 +1431,15 @@ cd_device_dbus_get_property (GDBusConnection *connection_, const gchar *sender,
 		retval = g_variant_new_uint32 (priv->owner);
 		goto out;
 	}
+	if (g_strcmp0 (property_name, CD_DEVICE_PROPERTY_PROFILING_INHIBITORS) == 0) {
+		bus_names = cd_inhibit_get_bus_names (priv->inhibit);
+		retval = g_variant_new_strv ((const gchar * const *) bus_names, -1);
+		goto out;
+	}
 
 	g_critical ("failed to get property %s", property_name);
 out:
+	g_strfreev (bus_names);
 	return retval;
 }
 
@@ -1466,6 +1509,7 @@ void
 cd_device_watch_sender (CdDevice *device, const gchar *sender)
 {
 	g_return_if_fail (CD_IS_DEVICE (device));
+	g_return_if_fail (sender != NULL);
 	device->priv->watcher_id = g_bus_watch_name (G_BUS_TYPE_SYSTEM,
 						     sender,
 						     G_BUS_NAME_WATCHER_FLAGS_NONE,
