@@ -36,7 +36,6 @@
 #include "cd-profile-array.h"
 #include "cd-profile.h"
 #include "cd-profile-store.h"
-#include "cd-sane-client.h"
 #include "cd-sensor-client.h"
 #include "cd-udev-client.h"
 
@@ -55,7 +54,6 @@ static CdDeviceDb *device_db = NULL;
 static CdUdevClient *udev_client = NULL;
 static CdSensorClient *sensor_client = NULL;
 #endif
-static CdSaneClient *sane_client = NULL;
 static CdConfig *config = NULL;
 static GPtrArray *sensors = NULL;
 static GHashTable *standard_spaces = NULL;
@@ -196,6 +194,7 @@ cd_main_add_profile (CdProfile *profile,
 static CdProfile *
 cd_main_create_profile (const gchar *sender,
 			const gchar *profile_id,
+			guint owner,
 			CdObjectScope scope,
 			GError **error)
 {
@@ -207,6 +206,7 @@ cd_main_create_profile (const gchar *sender,
 
 	/* create an object */
 	profile_tmp = cd_profile_new ();
+	cd_profile_set_owner (profile_tmp, owner);
 	cd_profile_set_id (profile_tmp, profile_id);
 	cd_profile_set_scope (profile_tmp, scope);
 
@@ -216,21 +216,25 @@ cd_main_create_profile (const gchar *sender,
 		goto out;
 
 	/* different persistent scope */
-	if (scope == CD_OBJECT_SCOPE_NORMAL) {
+	switch (scope) {
+	case CD_OBJECT_SCOPE_NORMAL:
 		g_debug ("CdMain: normal profile");
-	} else if ((scope & CD_OBJECT_SCOPE_TEMP) > 0) {
+		break;
+	case CD_OBJECT_SCOPE_TEMP:
 		g_debug ("CdMain: temporary profile");
 		/* setup DBus watcher */
 		cd_profile_watch_sender (profile_tmp, sender);
-	} else if ((scope & CD_OBJECT_SCOPE_DISK) > 0) {
+		break;
+	case CD_OBJECT_SCOPE_DISK:
 		g_debug ("CdMain: persistent profile");
 		g_set_error_literal (error,
 				     CD_MAIN_ERROR,
 				     CD_MAIN_ERROR_FAILED,
 				     "persistent profiles are no yet supported");
 		goto out;
-	} else {
+	default:
 		g_warning ("CdMain: Unsupported scope kind: %i", scope);
+		goto out;
 	}
 
 	/* success */
@@ -366,8 +370,9 @@ cd_main_device_auto_add_profiles (CdDevice *device)
 	/* try to add them */
 	for (i=0; i<array->len; i++) {
 		object_id_tmp = g_ptr_array_index (array, i);
-		profile_tmp = cd_profile_array_get_by_id (profiles_array,
-							  object_id_tmp);
+		profile_tmp = cd_profile_array_get_by_id_owner (profiles_array,
+								object_id_tmp,
+								cd_device_get_owner (device));
 		if (profile_tmp != NULL) {
 			cd_main_device_auto_add_profile (device, profile_tmp);
 			g_object_unref (profile_tmp);
@@ -442,9 +447,11 @@ cd_main_device_add (CdDevice *device,
 	}
 
 	/* profile is no longer valid */
-	g_signal_connect (device, "invalidate",
-			  G_CALLBACK (cd_main_device_invalidate_cb),
-			  NULL);
+	if (scope == CD_OBJECT_SCOPE_TEMP) {
+		g_signal_connect (device, "invalidate",
+				  G_CALLBACK (cd_main_device_invalidate_cb),
+				  NULL);
+	}
 
 	/* add to array */
 	cd_device_array_add (devices_array, device);
@@ -461,6 +468,7 @@ out:
 static CdDevice *
 cd_main_create_device (const gchar *sender,
 		       const gchar *device_id,
+		       guint owner,
 		       CdObjectScope scope,
 		       CdDeviceMode mode,
 		       GError **error)
@@ -473,6 +481,7 @@ cd_main_create_device (const gchar *sender,
 
 	/* create an object */
 	device_tmp = cd_device_new ();
+	cd_device_set_owner (device_tmp, owner);
 	cd_device_set_id (device_tmp, device_id);
 	cd_device_set_scope (device_tmp, scope);
 	cd_device_set_mode (device_tmp, mode);
@@ -481,7 +490,7 @@ cd_main_create_device (const gchar *sender,
 		goto out;
 
 	/* setup DBus watcher */
-	if ((scope & CD_OBJECT_SCOPE_TEMP) > 0) {
+	if (sender != NULL && scope == CD_OBJECT_SCOPE_TEMP) {
 		g_debug ("temporary device");
 		cd_device_watch_sender (device_tmp, sender);
 	}
@@ -613,8 +622,9 @@ cd_main_profile_auto_add_to_device (CdProfile *profile)
 	/* try to add them */
 	for (i=0; i<array->len; i++) {
 		device_id_tmp = g_ptr_array_index (array, i);
-		device_tmp = cd_device_array_get_by_id (devices_array,
-							device_id_tmp);
+		device_tmp = cd_device_array_get_by_id_owner (devices_array,
+							      device_id_tmp,
+							      cd_profile_get_owner (profile));
 		if (device_tmp != NULL) {
 			cd_main_device_auto_add_profile (device_tmp, profile);
 			g_object_unref (device_tmp);
@@ -732,11 +742,24 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 	GVariant *value = NULL;
 	gint fd = -1;
 	guint uid;
+	gint32 fd_handle = 0;
 	const gchar *metadata_key = NULL;
 	const gchar *metadata_value = NULL;
 	GDBusMessage *message;
 	GUnixFDList *fd_list;
 
+	/* get the owner of the message */
+	uid = cd_main_get_sender_uid (invocation, &error);
+	if (uid == G_MAXUINT) {
+		g_dbus_method_invocation_return_error (invocation,
+						       CD_MAIN_ERROR,
+						       CD_MAIN_ERROR_FAILED,
+						       "failed to get owner: %s",
+						       error->message);
+		g_error_free (error);
+		goto out;
+	}
+
 	/* return 'as' */
 	if (g_strcmp0 (method_name, "GetDevices") == 0) {
 
@@ -802,7 +825,9 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 		g_variant_get (parameters, "(&s)", &device_id);
 		g_debug ("CdMain: %s:FindDeviceById(%s)",
 			 sender, device_id);
-		device = cd_device_array_get_by_id (devices_array, device_id);
+		device = cd_device_array_get_by_id_owner (devices_array,
+							  device_id,
+							  uid);
 		if (device == NULL) {
 			g_dbus_method_invocation_return_error (invocation,
 							       CD_MAIN_ERROR,
@@ -851,7 +876,9 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 		g_variant_get (parameters, "(&s)", &device_id);
 		g_debug ("CdMain: %s:FindProfileById(%s)",
 			 sender, device_id);
-		profile = cd_profile_array_get_by_id (profiles_array, device_id);
+		profile = cd_profile_array_get_by_id_owner (profiles_array,
+							    device_id,
+							    uid);
 		if (profile == NULL) {
 			g_dbus_method_invocation_return_error (invocation,
 							       CD_MAIN_ERROR,
@@ -941,10 +968,13 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 			       &device_id,
 			       &scope_tmp,
 			       &iter);
-
 		g_debug ("CdMain: %s:CreateDevice(%s)", sender, device_id);
+
+		/* check it does not already exist */
 		scope = cd_object_scope_from_string (scope_tmp);
-		device = cd_device_array_get_by_id (devices_array, device_id);
+		device = cd_device_array_get_by_id_owner (devices_array,
+							  device_id,
+							  uid);
 		if (device != NULL) {
 			/* where we try to manually add an existing
 			 * virtual device, which means promoting it to
@@ -966,6 +996,7 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 		/* create device */
 		device = cd_main_create_device (sender,
 						device_id,
+						uid,
 						scope,
 						CD_DEVICE_MODE_UNKNOWN,
 						&error);
@@ -978,19 +1009,6 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 			goto out;
 		}
 
-		/* set the owner */
-		uid = cd_main_get_sender_uid (invocation, &error);
-		if (uid == G_MAXUINT) {
-			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
-							       "failed to get owner: %s",
-							       error->message);
-			g_error_free (error);
-			goto out;
-		}
-		cd_device_set_owner (device, uid);
-
 		/* set the properties */
 		while (g_variant_iter_next (iter, "{&s&s}",
 					    &prop_key, &prop_value)) {
@@ -1040,7 +1058,9 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 		g_variant_get (parameters, "(&o)", &device_id);
 		g_debug ("CdMain: %s:DeleteDevice(%s)",
 			 sender, device_id);
-		device = cd_device_array_get_by_id (devices_array, device_id);
+		device = cd_device_array_get_by_id_owner (devices_array,
+							  device_id,
+							  uid);
 		if (device == NULL) {
 			/* fall back to checking the object path */
 			device = cd_device_array_get_by_object_path (devices_array,
@@ -1075,7 +1095,9 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 		g_variant_get (parameters, "(&o)", &device_id);
 		g_debug ("CdMain: %s:DeleteProfile(%s)",
 			 sender, device_id);
-		profile = cd_profile_array_get_by_object_path (profiles_array, device_id);
+		profile = cd_profile_array_get_by_id_owner (profiles_array,
+							    device_id,
+							    uid);
 		if (profile == NULL) {
 			/* fall back to checking the object path */
 			profile = cd_profile_array_get_by_object_path (profiles_array,
@@ -1097,8 +1119,9 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 		goto out;
 	}
 
-	/* return 's' */
-	if (g_strcmp0 (method_name, "CreateProfile") == 0) {
+	/* return 'o' */
+	if (g_strcmp0 (method_name, "CreateProfile") == 0 ||
+	    g_strcmp0 (method_name, "CreateProfileWithFd") == 0) {
 
 		/* require auth */
 		ret = cd_main_sender_authenticated (invocation,
@@ -1106,14 +1129,28 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 		if (!ret)
 			goto out;
 
-		/* does already exist */
-		g_variant_get (parameters, "(&s&sa{ss})",
-			       &device_id,
-			       &scope_tmp,
-			       &iter);
-		g_debug ("CdMain: %s:CreateProfile(%s)", sender, device_id);
-		profile = cd_profile_array_get_by_id (profiles_array,
-						      device_id);
+		if (g_strcmp0 (g_variant_get_type_string (parameters),
+			       "(ssha{ss})") == 0) {
+			g_variant_get (parameters, "(&s&sha{ss})",
+				       &device_id,
+				       &scope_tmp,
+				       &fd_handle,
+				       &iter);
+			g_debug ("CdMain: %s:CreateProfileWithFd(%s,%i)",
+				 g_dbus_method_invocation_get_sender (invocation),
+				 device_id, fd_handle);
+		} else {
+			g_variant_get (parameters, "(&s&sa{ss})",
+				       &device_id,
+				       &scope_tmp,
+				       &iter);
+			g_debug ("CdMain: %s:CreateProfile(%s)", sender, device_id);
+		}
+
+		/* check it does not already exist */
+		profile = cd_profile_array_get_by_id_owner (profiles_array,
+							    device_id,
+							    uid);
 		if (profile != NULL) {
 			g_dbus_method_invocation_return_error (invocation,
 							       CD_MAIN_ERROR,
@@ -1122,9 +1159,12 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 							       device_id);
 			goto out;
 		}
+
+		/* create profile */
 		scope = cd_object_scope_from_string (scope_tmp);
 		profile = cd_main_create_profile (sender,
 						  device_id,
+						  uid,
 						  scope,
 						  &error);
 		if (profile == NULL) {
@@ -1133,19 +1173,6 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 			goto out;
 		}
 
-		/* set the owner */
-		uid = cd_main_get_sender_uid (invocation, &error);
-		if (uid == G_MAXUINT) {
-			g_dbus_method_invocation_return_error (invocation,
-							       CD_MAIN_ERROR,
-							       CD_MAIN_ERROR_FAILED,
-							       "failed to get owner: %s",
-							       error->message);
-			g_error_free (error);
-			goto out;
-		}
-		cd_profile_set_owner (profile, uid);
-
 		/* auto add profiles from the database */
 		cd_main_profile_auto_add_to_device (profile);
 
@@ -1153,7 +1180,7 @@ cd_main_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 		message = g_dbus_method_invocation_get_message (invocation);
 		fd_list = g_dbus_message_get_unix_fd_list (message);
 		if (fd_list != NULL && g_unix_fd_list_get_length (fd_list) == 1) {
-			fd = g_unix_fd_list_get (fd_list, 0, &error);
+			fd = g_unix_fd_list_get (fd_list, fd_handle, &error);
 			if (fd < 0) {
 				g_warning ("CdMain: failed to get fd from message: %s",
 					   error->message);
@@ -1331,6 +1358,7 @@ cd_main_add_disk_device (const gchar *device_id)
 
 	device = cd_main_create_device (NULL,
 					device_id,
+					0,
 					CD_OBJECT_SCOPE_DISK,
 					CD_DEVICE_MODE_VIRTUAL,
 					&error);
@@ -1469,8 +1497,9 @@ cd_main_setup_standard_space (const gchar *space,
 
 	/* depending on the prefix, find the profile */
 	if (g_str_has_prefix (search, "icc_")) {
-		profile = cd_profile_array_get_by_id (profiles_array,
-						      search);
+		profile = cd_profile_array_get_by_id_owner (profiles_array,
+							    search,
+							    0);
 	} else if (g_str_has_prefix (search, "/")) {
 		profile = cd_profile_array_get_by_filename (profiles_array,
 							    search);
@@ -1530,6 +1559,31 @@ out:
 	g_strfreev (spaces);
 }
 
+/**
+ * cd_main_colord_sane_refresh_cb:
+ **/
+static void
+cd_main_colord_sane_refresh_cb (GObject *source_object,
+				GAsyncResult *res,
+				gpointer user_data)
+{
+	GError *error = NULL;
+	GVariant *retval;
+
+	retval = g_dbus_connection_call_finish (connection,
+						res,
+						&error);
+	if (retval == NULL) {
+		g_warning ("failed to contact colord-sane: %s",
+			   error->message);
+		g_error_free (error);
+		goto out;
+	}
+out:
+	if (retval != NULL)
+		g_variant_unref (retval);
+}
+
 /**
  * cd_main_on_name_acquired_cb:
  **/
@@ -1605,12 +1659,18 @@ cd_main_on_name_acquired_cb (GDBusConnection *connection_,
 	/* add SANE devices */
 	ret = cd_config_get_boolean (config, "UseSANE");
 	if (ret) {
-		ret = cd_sane_client_refresh (sane_client, &error);
-		if (!ret) {
-			g_warning ("CdMain: failed to refresh SANE devices: %s",
-				    error->message);
-			g_error_free (error);
-		}
+		g_dbus_connection_call (connection,
+					"org.freedesktop.colord-sane",
+					"/org/freedesktop/colord_sane",
+					"org.freedesktop.colord.sane",
+					"Refresh",
+					NULL,
+					NULL,
+					G_DBUS_CALL_FLAGS_NONE,
+					-1,
+					NULL,
+					cd_main_colord_sane_refresh_cb,
+					NULL);
 	}
 
 	/* now we've got the profiles, setup the overrides */
@@ -1804,14 +1864,6 @@ main (int argc, char *argv[])
 						 g_str_equal,
 						 g_free,
 						 (GDestroyNotify) g_object_unref);
-
-	sane_client = cd_sane_client_new ();
-	g_signal_connect (sane_client, "added",
-			  G_CALLBACK (cd_main_client_device_added_cb),
-			  NULL);
-	g_signal_connect (sane_client, "removed",
-			  G_CALLBACK (cd_main_client_device_removed_cb),
-			  NULL);
 #ifdef HAVE_GUDEV
 	udev_client = cd_udev_client_new ();
 	g_signal_connect (udev_client, "device-added",
@@ -1924,8 +1976,6 @@ out:
 #endif
 	if (config != NULL)
 		g_object_unref (config);
-	if (sane_client != NULL)
-		g_object_unref (sane_client);
 	if (profile_store != NULL)
 		g_object_unref (profile_store);
 	if (mapping_db != NULL)
