@@ -28,7 +28,6 @@
 #include <glib-object.h>
 
 #include "cd-buffer.h"
-#include "cd-usb.h"
 
 static void
 cd_test_buffer_func (void)
@@ -46,37 +45,6 @@ cd_test_buffer_func (void)
 	g_assert_cmpint (cd_buffer_read_uint16_le (buffer), ==, 8192);
 }
 
-static void
-cd_test_usb_func (void)
-{
-	CdUsb *usb;
-	gboolean ret;
-	GError *error = NULL;
-
-	usb = cd_usb_new ();
-	g_assert (usb != NULL);
-	g_assert (!cd_usb_get_connected (usb));
-	g_assert (cd_usb_get_device_handle (usb) == NULL);
-
-	/* try to load */
-	ret = cd_usb_load (usb, &error);
-	g_assert_no_error (error);
-	g_assert (ret);
-
-	/* attach to the default mainloop */
-	ret = cd_usb_attach_to_context (usb, NULL, &error);
-	g_assert (ret);
-	g_assert_no_error (error);
-
-	/* connect to a non-existant device */
-	ret = cd_usb_connect (usb, 0xffff, 0xffff, 0x1, 0x1, &error);
-	g_assert (!ret);
-	g_assert_error (error, CD_USB_ERROR, CD_USB_ERROR_INTERNAL);
-	g_error_free (error);
-
-	g_object_unref (usb);
-}
-
 int
 main (int argc, char **argv)
 {
@@ -87,7 +55,6 @@ main (int argc, char **argv)
 	g_log_set_fatal_mask (NULL, G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL);
 
 	/* tests go here */
-	g_test_add_func ("/colord/usb", cd_test_usb_func);
 	g_test_add_func ("/colord/buffer", cd_test_buffer_func);
 	return g_test_run ();
 }
