@@ -26,8 +26,8 @@
 #include <gusb.h>
 
 /* device constants */
-#define	CH_USB_VID				0x04d8
-#define	CH_USB_PID				0xf8da
+#define	CH_USB_VID				0x273f
+#define	CH_USB_PID				0x1001
 #define	CH_USB_CONFIG				0x0001
 #define	CH_USB_INTERFACE			0x0000
 #define	CH_USB_HID_EP				0x0001
@@ -35,6 +35,10 @@
 #define	CH_USB_HID_EP_OUT			(CH_USB_HID_EP | 0x00)
 #define	CH_USB_HID_EP_SIZE			64
 
+/* old device constants */
+#define	CH_USB_VID_LEGACY			0x04d8
+#define	CH_USB_PID_LEGACY			0xf8da
+
 /* the default timeout in client tools */
 #define CH_DEVICE_USB_TIMEOUT			5000 /* ms */
 
@@ -311,12 +315,12 @@
  * CH_CMD_TAKE_READINGS:
  *
  * Take a reading taking into account just dark offsets.
- * All of @red, @green and @blue are _signed_ values.
+ * All of @red, @green and @blue are packed float values.
  *
  * This command is useful if you want to do an ambient reading.
  *
  * IN:  [1:cmd]
- * OUT: [1:retval][1:cmd][2:red][2:green][2:blue]
+ * OUT: [1:retval][1:cmd][4:red][4:green][4:blue]
  *
  * This command is only available in firmware mode.
  **/
@@ -431,10 +435,10 @@
 /**
  * CH_CMD_GET_PRE_SCALE:
  *
- * Get the pre XYZ scaling constant.
+ * Get the pre XYZ scaling constant. @scale is a packed float.
  *
  * IN:  [1:cmd]
- * OUT: [1:retval][1:cmd][2:scale]
+ * OUT: [1:retval][1:cmd][4:scale]
  *
  * This command is only available in firmware mode.
  **/
@@ -443,9 +447,9 @@
 /**
  * CH_CMD_SET_PRE_SCALE:
  *
- * Set the pre XYZ scaling constant.
+ * Set the pre XYZ scaling constant. @scale is a packed float.
  *
- * IN:  [1:cmd][2:scale]
+ * IN:  [1:cmd][4:scale]
  * OUT: [1:retval][1:cmd]
  *
  * This command is only available in firmware mode.
@@ -455,10 +459,10 @@
 /**
  * CH_CMD_GET_POST_SCALE:
  *
- * Get the post XYZ scaling constant.
+ * Get the post XYZ scaling constant. @scale is a packed float.
  *
  * IN:  [1:cmd]
- * OUT: [1:retval][1:cmd][2:scale]
+ * OUT: [1:retval][1:cmd][4:scale]
  *
  * This command is only available in firmware mode.
  **/
@@ -467,9 +471,9 @@
 /**
  * CH_CMD_SET_POST_SCALE:
  *
- * Set the post XYZ scaling constant.
+ * Set the post XYZ scaling constant. @scale is a packed float.
  *
- * IN:  [1:cmd][2:scale]
+ * IN:  [1:cmd][4:scale]
  * OUT: [1:retval][1:cmd]
  *
  * This command is only available in firmware mode.
@@ -580,6 +584,33 @@
  **/
 #define	CH_CMD_GET_PCB_ERRATA			0x33
 
+/**
+ * CH_CMD_SET_REMOTE_HASH:
+ *
+ * Sets the remote SHA1 hash of the profile. This is designed to
+ * be used by the calibration program to indicate the key which allows
+ * the completed profile to be found from a public web service.
+ *
+ * IN:  [1:cmd][20:sha1_hash]
+ * OUT: [1:retval][1:cmd]
+ *
+ * This command is available in firmware mode.
+ **/
+#define	CH_CMD_SET_REMOTE_HASH			0x34
+
+/**
+ * CH_CMD_GET_REMOTE_HASH:
+ *
+ * Gets the remote hash which is used to get the last profile saved
+ * to a public web service.
+ *
+ * IN:  [1:cmd]
+ * OUT: [1:retval][1:cmd][20:sha1_hash]
+ *
+ * This command is available in firmware mode.
+ **/
+#define	CH_CMD_GET_REMOTE_HASH			0x35
+
 /* secret code */
 #define	CH_WRITE_EEPROM_MAGIC			"Un1c0rn2"
 
@@ -694,4 +725,15 @@ gboolean	 ch_device_write_command_finish	(GUsbDevice	*device,
 						 GAsyncResult	*res,
 						 GError		**error);
 
+
+/* SHA1 hash */
+typedef struct {
+	guint8	bytes[20];
+} ChSha1;
+
+gchar		*ch_sha1_to_string		(const ChSha1		*sha1);
+gboolean	 ch_sha1_parse			(const gchar		*value,
+						 ChSha1			*sha1,
+						 GError			**error);
+
 #endif
